#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2019 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>Custom Font Tools>Custom Font Metallic Effects v0.01


from gimpfu import *
#from array import array
#import math
import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
darkestgrey = gimpcolor.RGB(20, 20, 20)
verydarkgrey = gimpcolor.RGB(50, 50, 50)
darkgrey = gimpcolor.RGB(75, 75, 75)
midgrey2 = gimpcolor.RGB(105, 105, 105)
midgrey = gimpcolor.RGB(125, 125, 125)
lightgrey = gimpcolor.RGB(170, 170, 170)
verylightgrey = gimpcolor.RGB(222, 222, 222)
red = gimpcolor.RGB(220,10,10)	

# Some global metallic colours we may need for creating gradients

lightsilver = gimpcolor.RGB(204, 202, 202)
darksilver = gimpcolor.RGB(46, 43, 43)
lightgold = gimpcolor.RGB(250, 233, 49)
darkgold = gimpcolor.RGB(240, 183, 42)
lightcopper = gimpcolor.RGB(212, 125, 1)
darkcopper = gimpcolor.RGB(177, 106, 58)
lightbrass = gimpcolor.RGB(253, 242, 91)
darkbrass = gimpcolor.RGB(220 , 118, 20)
lightbronze = gimpcolor.RGB(104, 75, 33)
darkbronze = gimpcolor.RGB(83, 69, 42)


def do_stuff(image, layer, metal_texture, metal_colour, retain_original, do_gradient, fill_gradient, bump_depth, light_amount, compass, effect_elevation, do_sheen, apply_lighting, shade_of_grey, effect_mode_ptr, artifact_radius, artifact_delta, final_bump_depth):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)	
    do_metal = True
    loop_ptr = 1
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    layer_names_list = image.layers
	
    if not (do_metal == False):
        # turn off visibility of layers
        visi_loop, first_char_layer  = 0, -1
        multi_layer_list = "No non-character layers."
        pitted_drawable_name = "None"
        num_multi_layers,other_layers = 0,0
        while visi_loop < num_layers:
	        # get info about layer we are working on
            layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            layer_name = layer_name.decode('utf-8')
            if len(layer_name) == 1 or len(layer_name) == 5 and ".png" in layer_name:
                if first_char_layer < 0: 
                    first_char_layer = visi_loop
                    first_char_layer_name = layer_name
            else:
                pdb.gimp_image_select_item(image, 2, layer_names_list[visi_loop])
                is_empty = pdb.gimp_selection_is_empty(image)
                if is_empty == False:
                    # make sure that existing layers are positioned at the origin of the image
                    layer_names_list[visi_loop].set_offsets(0, 0)
                    if other_layers == 0:
                        
                        multi_layer_list = [layer_name]
                    else:
                        
                        multi_layer_list.append(layer_name)
                    other_layers += 1
            pdb.gimp_item_set_visible(layer_names_list[visi_loop],False)
            visi_loop += 1
			
        if other_layers>0: pitted_drawable_name = multi_layer_list[0]
	
	
    while loop >= 0:
	    # get info about layer we are working on
        layer_id = layer_ids[loop]
        layer_name = layer_names_list[loop]
	
        # cater for unicode characters above normal range
        display_layer_name =  pdb.gimp_item_get_name(layer_names_list[loop])
        display_layer_name = display_layer_name.decode('utf-8')	
	
        # check we have the layer id of the currently active layer
        pdb.gimp_image_set_active_layer(image, layer_name)
        layer_id = pdb.gimp_image_get_active_layer(image)

        # check to see if the current layer is a custom font character layer
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_id, display_layer_name)
		
        if test_for_custom_layer:

            # ensure we have the active layer and make it visible
            active_layer_id = pdb.gimp_image_get_active_layer(image)
            pdb.gimp_item_set_visible(active_layer_id, True)
			
            if do_metal:
                active_layer = image.active_layer
                active_layer_id = proc_metal(image, active_layer, shade_of_grey, fill_gradient, do_gradient, metal_colour, metal_texture, do_sheen, bump_depth, effect_elevation, effect_mode_ptr, retain_original, compass, light_amount, apply_lighting, artifact_radius, artifact_delta, other_layers, pitted_drawable_name, final_bump_depth)
                #loop_ptr += 2 # temp temp temp
				
				
            if (num_layers - loop_ptr) > 0: pdb.gimp_item_set_visible(active_layer_id, False)
			
            num_layers, layer_ids = pdb.gimp_image_get_layers(image)
			
        loop_ptr += 1
        loop = num_layers - loop_ptr
	
    if first_char_layer_name <> "none":
        first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
        pdb.gimp_item_set_visible(first_char_layer_id, True)
        pdb.gimp_image_set_active_layer(image,first_char_layer_id)
		
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	       

######################################################################################################################################

def proc_do_bump(image, other_layers, bumpmap_layer_id, bumpmap_layer_name, active_layer, black, white, bumpmap_width, waterlevel, ambient, effect_elevation, effect_azimuth, bevel_mode_ptr):
    if other_layers > 0:
        work_layer = active_layer
        work_layer_name = work_layer.name
        width = work_layer.width
        height = work_layer.height
        bumpmap_width = bumpmap_width * 0.65
        if bumpmap_width < 1: bumpmap_width = 1
		
        if waterlevel > 0:
            waterlevel = waterlevel/100
            waterlevel = 1.0 - waterlevel
        else:
            waterlevel = 0.0
			
        if ambient > 0:
            ambient = ambient/100
            ambient = 1.0 - ambient
        else:
            ambient = 0.0        

        pdb.plug_in_bump_map(image, work_layer, bumpmap_layer_id, effect_azimuth, effect_elevation, bumpmap_width, 0, 0, waterlevel, ambient, True, False, 0)
		
        active_layer = image.active_layer 
        return active_layer
        
######################################################################################################################################

def proc_metal(image, active_layer, shade_of_grey, fill_gradient, do_gradient, custom_colour, metal_texture, do_sheen, bump_depth, effect_elevation, effect_mode_ptr, retain_original, compass, light_amount, apply_lighting, artifact_radius, artifact_delta, other_layers, pitted_drawable_name, final_bump_depth):

    effect_azimuth, light_pos_x, light_pos_y, light_dir_x, light_dir_y = fn_compass(compass, light_amount)

    work_layer = active_layer # ie character layer id
    work_layer_name = work_layer.name # ie character layer name
    width = work_layer.width
    height = work_layer.height
	
    #retain_original = True ##### TEMP Value
		
    current_fg = pdb.gimp_context_get_foreground() # make a note of the current foreground colour
	
    # set bumpmap 
    bump_depth = bump_depth * 0.65
    if bump_depth < 1.0: bump_depth = 1.0
    waterlevel = 0
    ambient = 0.5
    if final_bump_depth>0:
        final_bump_depth = final_bump_depth * 0.65
        if final_bump_depth < 1.0: final_bump_depth = 1.0
	
    if metal_texture == 0: #Glitter
	
        if not retain_original:
            temp_layer = gimp.Layer(image, "Temp Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the darker layer for white bases
            image.add_layer(temp_layer, -1)
            metal_grey = fn_get_bg_bump_col(shade_of_grey)
            pdb.gimp_context_set_foreground(metal_grey)
            pdb.gimp_image_select_item(image, 2, work_layer) # get character outline
            pdb.gimp_drawable_edit_fill(temp_layer, 0) # fill character outline
            pdb.gimp_selection_none(image)
            pdb.gimp_drawable_edit_clear(work_layer)
            work_layer = pdb.gimp_image_merge_visible_layers(image,0)
    
	
	
        pdb.gimp_context_set_foreground(black) 
        metal_layer = gimp.Layer(image, "Metal Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the metallic layer
        image.add_layer(metal_layer, -1)
        pdb.gimp_drawable_fill(metal_layer, 0) # fill entire layer
	
        pdb.gimp_image_select_item(image, 2, work_layer)
        # get y co-ordinate values for blending
        current_selection, cs_x1, cs_y1, cs_x2, cs_y2, cs_width, cs_height  = fn_selection_info(image)
        blend_y0 = cs_y1
        blend_y1 = cs_y2
        sel_blur_factor = cs_height * 0.05 # base initial blur on height of actual character within layer
	
        pdb.gimp_context_set_foreground(white)
        pdb.gimp_drawable_edit_fill(metal_layer, 0) # fill character outline
	
        pdb.gimp_selection_none(image)
	
        #bump_depth = bump_depth * 0.65
        #if bump_depth < 1.0: bump_depth = 1.0
        #waterlevel = 0
        #ambient = 0.5	
	
        # make a copy of the original character so that we can use it later to meld with the newly applied lighting effects
        copy_of_metal = pdb.gimp_layer_copy(metal_layer, True)
        image.add_layer(copy_of_metal, -1)
        pdb.gimp_item_set_visible(copy_of_metal, False)
        pdb.gimp_item_set_name(copy_of_metal,"Copy of Metal")
        pdb.plug_in_gauss(image, copy_of_metal, (sel_blur_factor), (sel_blur_factor), 1)
        final_copy = pdb.gimp_layer_copy(copy_of_metal, True)
        image.add_layer(final_copy, -1)
        pdb.gimp_item_set_name(final_copy,"Final Copy")
        pdb.gimp_drawable_edit_clear(metal_layer)
	
        if retain_original:
	
            # make a copy of the original character and desaturate it ready for bumpmapping
            copy = pdb.gimp_edit_copy(work_layer)
            floater = pdb.gimp_edit_paste(metal_layer ,True)
            pdb.gimp_floating_sel_anchor(floater)
            pdb.gimp_drawable_desaturate(metal_layer,0)			
			
            temp_copy = pdb.gimp_layer_copy(metal_layer, True)
            image.add_layer(temp_copy, -1)
            pdb.gimp_item_set_name(temp_copy,"Temp Copy")
            pdb.gimp_item_set_visible(temp_copy, True)
            random.seed()
            amount = random.randrange(100,10000000,1)
			
            pdb.plug_in_hsv_noise(image,temp_copy, 1,180,255,255)
            pdb.plug_in_hsv_noise(image,temp_copy, 1,180,255,255)
            pdb.gimp_drawable_desaturate(temp_copy,0)
			
            mode = 45 # soft light
            temp_copy.mode = mode
            metal_layer = pdb.gimp_image_merge_down(image, temp_copy, 0)
		
        else:
        
            # fill the character outline with colour ready for bumpmapping
            pdb.gimp_image_select_item(image, 2, work_layer)
            pdb.gimp_context_set_foreground(verylightgrey)
            pdb.gimp_drawable_edit_fill(metal_layer, 0) # fill character outline with grey
            random.seed()
            pdb.gimp_selection_none(image)
            pdb.plug_in_hsv_noise(image,metal_layer, 1,180,255,255)
            pdb.plug_in_hsv_noise(image,metal_layer, 1,180,255,255)
            pdb.gimp_drawable_desaturate(metal_layer,0)
	
	
        pdb.plug_in_bump_map(image, metal_layer, copy_of_metal, effect_azimuth, effect_elevation, bump_depth, 0, 0, waterlevel, ambient, True, False, 0)
		
        active_layer = image.active_layer 
        metal_colour = active_layer
	
	
        # REM: Set lighting effect to metal_layer using copy_of_metal for bump map
        # Set variables for proc_lighting
        bump_map = metal_layer
        #if retain_original: bump_map = metal_layer
        env_map = None
        bump_map_flag = True
        env_map_flag = False
        bump_map_curve = 0
        light_source = 1
        light_colour = (255,255,255)
        #light_pos_x = -1.00
        #light_pos_y = -1.00
        light_pos_z = 1.00
        #light_dir_x = -1.50
        #light_dir_y = -1.50
        light_dir_z = 1.00
        light_glow = 0.80 # 0.60
        light_bright =  0.50 # 0.50
        light_diffuse_reflec = 3.00 #2.80
        light_shiny = 0.20 # 1.00 0.10
        light_polished = 30 #27 20
        anti_aliasing = True
        new_light_image = False
        transparent_bg = False
	
        if apply_lighting:
	        #REM: Call Lighting plugin
            proc_lighting(image, metal_layer, bump_map, env_map, bump_map_flag, env_map_flag, bump_map_curve, light_source, light_colour, 
            light_pos_x, light_pos_y, light_pos_z,
            light_dir_x, light_dir_y, light_dir_z, 
            light_glow, light_bright, light_diffuse_reflec, light_shiny, light_polished, 
            anti_aliasing, new_light_image, transparent_bg)
	
        temp = pdb.gimp_displays_flush()
		
        #Rem Prepare curves
        random.seed()
        curves_list = [0,0,44,82,78,31,116,209,141,76,165,255]
        random_curves_list = [0,0,44,82,78,31,116,random.randrange(209,254,1),141,random.randrange(7,76,1),165,255]
        #Rem Add curves    
        if do_sheen:
            pdb.gimp_curves_spline(metal_layer, 0, len(random_curves_list), random_curves_list)
        else:            
            pdb.gimp_curves_spline(metal_layer, 0, len(curves_list), curves_list)		
	
        pdb.gimp_drawable_edit_clear(copy_of_metal)	
	
        copy_layer = pdb.gimp_layer_copy(metal_layer, TRUE)
        temp = pdb.gimp_image_insert_layer(image, copy_layer, None, -1)
        pdb.plug_in_autocrop_layer(image, copy_layer)
        pdb.gimp_drawable_edit_clear(copy_layer)
        copy_layer.fill(1)
        pdb.gimp_image_select_item(image, 2, copy_layer)
        # get y co-ordinate values for blending
        current_selection, cs_x1, cs_y1, cs_x2, cs_y2  = pdb.gimp_selection_bounds(image)
        blend_x0, blend_y0, blend_x1, blend_y1 = fn_gradient_direction(compass, width, cs_x1, cs_y1, cs_x2, cs_y2)
        pdb.gimp_selection_none(image)
        image.remove_layer(copy_layer)
        gradient_mode = fn_set_bevel_mode(effect_mode_ptr)
	
	
        # fill the selection with the gradient
        if do_gradient:
            pdb.gimp_context_set_gradient(fill_gradient)
            pdb.gimp_drawable_edit_gradient_fill(copy_of_metal, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
        else:
	        # REM Prepare colours for custom gradient
            #Add colour and layer mode information
            gradient_mode, name_colour = fn_set_metal_colour(custom_colour)
            pdb.gimp_context_set_gradient_fg_bg_rgb()				
            # Do blending
            pdb.gimp_drawable_edit_gradient_fill(copy_of_metal, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)	
	
        pdb.gimp_item_set_visible(copy_of_metal, True)
        copy_of_metal.mode = gradient_mode
        pdb.gimp_drawable_edit_clear(work_layer)
        work_layer = pdb.gimp_image_merge_visible_layers(image,0)
        if final_bump_depth>0: pdb.plug_in_bump_map(image, work_layer, final_copy, effect_azimuth, effect_elevation, final_bump_depth, 0, 0, waterlevel, ambient, True, False, 0)	
        pdb.plug_in_sel_gauss(image, work_layer, artifact_radius, artifact_delta)
		
        image.remove_layer(final_copy)
		
		
    if metal_texture == 1: # Bright
	
        old_gradient = pdb.gimp_context_get_gradient()
        if not retain_original:
            temp_layer = gimp.Layer(image, "Temp Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the darker layer for white bases
            image.add_layer(temp_layer, -1)
            metal_grey = fn_get_bg_bump_col(shade_of_grey)
            pdb.gimp_context_set_foreground(metal_grey)
            pdb.gimp_image_select_item(image, 2, work_layer) # get character outline
            pdb.gimp_drawable_edit_fill(temp_layer, 0) # fill character outline
            pdb.gimp_selection_none(image)
            pdb.gimp_drawable_edit_clear(work_layer)
            work_layer = pdb.gimp_image_merge_visible_layers(image,0)
    	
        pdb.gimp_context_set_foreground(black) 
        metal_layer = gimp.Layer(image, "Metal Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the metallic layer
        image.add_layer(metal_layer, -1)
        pdb.gimp_drawable_fill(metal_layer, 0) # fill entire layer
	
        pdb.gimp_image_select_item(image, 2, work_layer)
        # get y co-ordinate values for blending
        current_selection, cs_x1, cs_y1, cs_x2, cs_y2, cs_width, cs_height  = fn_selection_info(image)
        blend_y0 = cs_y1
        blend_y1 = cs_y2
        sel_blur_factor = cs_height * 0.05 # base initial blur on height of actual character within layer
	
        pdb.gimp_context_set_foreground(white)
        pdb.gimp_drawable_edit_fill(metal_layer, 0) # fill character outline
	
        pdb.gimp_selection_none(image)
	
        #bump_depth = bump_depth * 0.65
        #if bump_depth < 1.0: bump_depth = 1.0
        #waterlevel = 0
        #ambient = 0.5
	
	
        # make a copy of the original character so that we can use it later to meld with the newly applied lighting effects
        copy_of_metal = pdb.gimp_layer_copy(metal_layer, True)
        image.add_layer(copy_of_metal, -1)
        pdb.gimp_item_set_visible(copy_of_metal, False)
        pdb.gimp_item_set_name(copy_of_metal,"Copy of Metal")
        pdb.plug_in_gauss(image, copy_of_metal, (sel_blur_factor), (sel_blur_factor), 1)
        final_copy = pdb.gimp_layer_copy(copy_of_metal, True)
        image.add_layer(final_copy, -1)
        pdb.gimp_item_set_name(final_copy,"Final Copy")
        pdb.gimp_drawable_edit_clear(metal_layer)
		
        if retain_original:
            # make a copy of the original character and desaturate it ready for bumpmapping
            copy = pdb.gimp_edit_copy(work_layer)
            floater = pdb.gimp_edit_paste(metal_layer ,True)
            pdb.gimp_floating_sel_anchor(floater)
            pdb.gimp_drawable_desaturate(metal_layer,0)
			
			
            temp_copy = pdb.gimp_layer_copy(metal_layer, True)
            image.add_layer(temp_copy, -1)
            pdb.gimp_item_set_name(temp_copy,"Temp Copy")
            pdb.gimp_item_set_visible(temp_copy, True)
            random.seed()
            amount = random.randrange(100,10000000,1)
			
            mode = 45 # soft light
            temp_copy.mode = mode
            metal_layer = pdb.gimp_image_merge_down(image, temp_copy, 0)
		
        else:
            pdb.gimp_image_select_item(image, 2, work_layer)
		
            pdb.gimp_drawable_edit_fill(metal_layer, 0) # fill character outline
        copy_layer = pdb.gimp_layer_copy(metal_layer, TRUE)
        temp = pdb.gimp_image_insert_layer(image, copy_layer, None, -1)
        pdb.gimp_selection_none(image)
        pdb.gimp_image_set_active_layer(image, copy_layer)
        pdb.plug_in_autocrop_layer(image, copy_layer)
        pdb.gimp_drawable_edit_fill(copy_layer, 0) # fill character outline
        pdb.gimp_image_select_item(image, 2, copy_layer)
        # get y co-ordinate values for blending
        current_selection, cs_x1, cs_y1, cs_x2, cs_y2  = pdb.gimp_selection_bounds(image)
        blend_x0, blend_y0, blend_x1, blend_y1 = fn_gradient_direction(compass, width, cs_x1, cs_y1, cs_x2, cs_y2)
        pdb.gimp_selection_none(image)
        image.remove_layer(copy_layer)
        gradient_mode = fn_set_bevel_mode(effect_mode_ptr)		
		
        if retain_original:
            temp_layer = gimp.Layer(image, "Temp Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the darker layer for white bases
            image.add_layer(temp_layer, -1)
        else:
            pdb.gimp_drawable_edit_clear(metal_layer)
	
            pdb.gimp_image_select_item(image, 2, work_layer)
            pdb.gimp_context_set_foreground(lightgrey)

        pdb.gimp_context_set_background(darkestgrey)
        pdb.gimp_context_set_gradient_fg_bg_rgb()
		
        if retain_original:
            temp_layer.mode = gradient_mode
            pdb.gimp_drawable_edit_gradient_fill(temp_layer, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
            metal_layer = pdb.gimp_image_merge_down(image, temp_layer, 0)
        else:
            pdb.gimp_drawable_edit_gradient_fill(metal_layer, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
			
        pdb.gimp_selection_none(image)
		
        pdb.plug_in_bump_map(image, metal_layer, copy_of_metal, effect_azimuth, effect_elevation, bump_depth, 0, 0, waterlevel, ambient, True, False, 0)
		
        #pdb.plug_in_sel_gauss(image, metal_layer, artifact_radius, artifact_delta)
		
		
		
        # REM: Set lighting effect to metal_layer using copy_of_metal for bump map
        # Set variables for proc_lighting
        bump_map = metal_layer
        #if retain_original: bump_map = metal_layer
        env_map = None
        bump_map_flag = True
        env_map_flag = False
        bump_map_curve = 0
        light_source = 1
        light_colour = (255,255,255)
        #light_pos_x = -1.00
        #light_pos_y = -1.00
        light_pos_z = 1.00
        #light_dir_x = -1.50
        #light_dir_y = -1.50
        light_dir_z = 1.00
        light_glow = 0.80 # 0.60
        light_bright =  0.50 # 0.50
        light_diffuse_reflec = 3.00 #2.80
        light_shiny = 0.20 # 1.00 0.10
        light_polished = 30 #27 20
        anti_aliasing = True
        new_light_image = False
        transparent_bg = False
		
        if apply_lighting:
	
	        #REM: Call Lighting plugin
            proc_lighting(image, metal_layer, bump_map, env_map, bump_map_flag, env_map_flag, bump_map_curve, light_source, light_colour, 
            light_pos_x, light_pos_y, light_pos_z,
            light_dir_x, light_dir_y, light_dir_z, 
            light_glow, light_bright, light_diffuse_reflec, light_shiny, light_polished, 
            anti_aliasing, new_light_image, transparent_bg)
	
        temp = pdb.gimp_displays_flush()
		
		
        #curves_list = [0, 0, 15, 255, 60, 10, 90, 230, 150, 125, 170, 255, 225, 195, 255, 255]
        #random_curves_list = [0, 0, random.randrange(30,50,1), 255, random.randrange(50,80,1), 0, random.randrange(80,100, 1), 255, random.randrange(100,140,1), 0, random.randrange(140,180,1), 255, random.randrange(180,240,1), 0, 255, 255]
        curves_list = [0,0,9,88,66,255,105,57,129,254,150,73,182,253,194,140,212,236,233,190,255,255]
        random_curves_list = [0,0,9,88,66,random.randrange(210,255,1),random.randrange(85,120,1),57,129,254,150,73,182,random.randrange(200,253,1),194,140,212,236,233,190,255,255]
		
		
        if do_sheen:
            pdb.gimp_curves_spline(metal_layer, 0, len(random_curves_list), random_curves_list)
        else:
            pdb.gimp_curves_spline(metal_layer, 0, len(curves_list), curves_list)
            
			
        pdb.gimp_drawable_desaturate(metal_layer,0)
			
        # fill the selection with the gradient
        if do_gradient:
            pdb.gimp_context_set_gradient(fill_gradient)
            pdb.gimp_drawable_edit_gradient_fill(copy_of_metal, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
        else:
	        # REM Prepare colours for custom gradient
            #Add colour and layer mode information
            gradient_mode, name_colour = fn_set_metal_colour(custom_colour)
            pdb.gimp_context_set_gradient_fg_bg_rgb()
				
        # Do blending
            pdb.gimp_drawable_edit_gradient_fill(copy_of_metal, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)	
	
        pdb.gimp_item_set_visible(copy_of_metal, True)
        copy_of_metal.mode = gradient_mode
        pdb.gimp_drawable_edit_clear(work_layer)
        work_layer = pdb.gimp_image_merge_visible_layers(image,0)
        if final_bump_depth>0: pdb.plug_in_bump_map(image, work_layer, final_copy, effect_azimuth, effect_elevation, final_bump_depth, 0, 0, waterlevel, ambient, True, False, 0)	
        pdb.plug_in_sel_gauss(image, work_layer, artifact_radius, artifact_delta)
        image.remove_layer(final_copy)
			
    if metal_texture == 2: # Verdigris
	
        if not retain_original:
            temp_layer = gimp.Layer(image, "Temp Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the darker layer for white bases
            image.add_layer(temp_layer, -1)
            metal_grey = fn_get_bg_bump_col(shade_of_grey)
            pdb.gimp_context_set_foreground(metal_grey)
            pdb.gimp_image_select_item(image, 2, work_layer) # get character outline
            pdb.gimp_drawable_edit_fill(temp_layer, 0) # fill character outline
            pdb.gimp_selection_none(image)
            pdb.gimp_drawable_edit_clear(work_layer)
            work_layer = pdb.gimp_image_merge_visible_layers(image,0)
    
	
	
        pdb.gimp_context_set_foreground(black) 
        metal_layer = gimp.Layer(image, "Metal Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the metallic layer
        image.add_layer(metal_layer, -1)
        pdb.gimp_drawable_fill(metal_layer, 0) # fill entire layer
	
        pdb.gimp_image_select_item(image, 2, work_layer)
        # get y co-ordinate values for blending
        current_selection, cs_x1, cs_y1, cs_x2, cs_y2, cs_width, cs_height  = fn_selection_info(image)
        blend_y0 = cs_y1
        blend_y1 = cs_y2
        sel_blur_factor = cs_height * 0.05 # base initial blur on height of actual character within layer
	
        pdb.gimp_context_set_foreground(white)
        pdb.gimp_drawable_edit_fill(metal_layer, 0) # fill character outline
	
        pdb.gimp_selection_none(image)
	
        #bump_depth = bump_depth * 0.65
        #if bump_depth < 1.0: bump_depth = 1.0
        #waterlevel = 0
        #ambient = 0.5
	
	
        # make a copy of the original character so that we can use it later to meld with the newly applied lighting effects
        copy_of_metal = pdb.gimp_layer_copy(metal_layer, True)
        image.add_layer(copy_of_metal, -1)
        pdb.gimp_item_set_visible(copy_of_metal, False)
        pdb.gimp_item_set_name(copy_of_metal,"Copy of Metal")
        pdb.plug_in_gauss(image, copy_of_metal, (sel_blur_factor), (sel_blur_factor), 1)
        final_copy = pdb.gimp_layer_copy(copy_of_metal, True)
        image.add_layer(final_copy, -1)
        pdb.gimp_item_set_name(final_copy,"Final Copy")
        pdb.gimp_drawable_edit_clear(metal_layer)
		
        if retain_original:
	
            # make a copy of the original character and desaturate it ready for bumpmapping
            copy = pdb.gimp_edit_copy(work_layer)
            floater = pdb.gimp_edit_paste(metal_layer ,True)
            pdb.gimp_floating_sel_anchor(floater)
            pdb.gimp_drawable_desaturate(metal_layer,0)
			
			
            temp_copy = pdb.gimp_layer_copy(metal_layer, True)
            image.add_layer(temp_copy, -1)
            pdb.gimp_item_set_name(temp_copy,"Temp Copy")
            pdb.gimp_item_set_visible(temp_copy, True)
            random.seed()
            amount = random.randrange(100,10000000,1)


            pdb.gimp_image_select_item(image, 2, work_layer)
            pdb.gimp_context_set_foreground(verylightgrey)
            pdb.gimp_drawable_edit_fill(metal_layer, 0) # fill character outline with grey
            random.seed()
            pdb.gimp_selection_none(image)
            
            pdb.plug_in_plasma(image,metal_layer, random.randrange(1000000, 2000000), 7)
            pdb.gimp_image_select_item(image, 2, work_layer)
            pdb.gimp_selection_invert(image)
            successful = pdb.gimp_edit_cut(metal_layer)
            pdb.gimp_selection_none(image)

            pdb.gimp_drawable_desaturate(metal_layer,0)
			
            mode = 50 # linear light
            temp_copy.mode = mode
            metal_layer = pdb.gimp_image_merge_down(image, temp_copy, 0)
		
        else:
	
            # fill the character outline with colour ready for bumpmapping
            pdb.gimp_image_select_item(image, 2, work_layer)
            pdb.gimp_context_set_foreground(verylightgrey)
            pdb.gimp_drawable_edit_fill(metal_layer, 0) # fill character outline with grey
            random.seed()
            pdb.gimp_selection_none(image)
            
            pdb.plug_in_plasma(image,metal_layer, random.randrange(1000000, 2000000), 7)
            pdb.gimp_image_select_item(image, 2, work_layer)
            pdb.gimp_selection_invert(image)
            successful = pdb.gimp_edit_cut(metal_layer)
            pdb.gimp_selection_none(image)
            pdb.gimp_drawable_desaturate(metal_layer,0)
			
        pdb.plug_in_bump_map(image, metal_layer, copy_of_metal, effect_azimuth, effect_elevation, bump_depth, 0, 0, waterlevel, ambient, True, False, 0)
        pdb.plug_in_bump_map(image, metal_layer, copy_of_metal, effect_azimuth, effect_elevation, bump_depth, 0, 0, waterlevel, ambient, True, False, 0)
		
        active_layer = image.active_layer 
        #pdb.plug_in_sel_gauss(image, active_layer, artifact_radius, artifact_delta)
        metal_colour = active_layer
	
	
        # REM: Set lighting effect to metal_layer using copy_of_metal for bump map
        # Set variables for proc_lighting
        bump_map = metal_layer
        #if retain_original: bump_map = metal_layer
        env_map = None
        bump_map_flag = True
        env_map_flag = False
        bump_map_curve = 0
        light_source = 1
        light_colour = (255,255,255)
        #light_pos_x = -1.00
        #light_pos_y = -1.00
        light_pos_z = 1.00
        #light_dir_x = -1.50
        #light_dir_y = -1.50
        light_dir_z = 1.00
        light_glow = 0.80 # 0.60
        light_bright =  0.50 # 0.50
        light_diffuse_reflec = 3.00 #2.80
        light_shiny = 0.20 # 1.00 0.10
        light_polished = 30 #27 20
        anti_aliasing = True
        new_light_image = False
        transparent_bg = False
	
        if apply_lighting:
	        #REM: Call Lighting plugin
            proc_lighting(image, metal_layer, bump_map, env_map, bump_map_flag, env_map_flag, bump_map_curve, light_source, light_colour, 
            light_pos_x, light_pos_y, light_pos_z,
            light_dir_x, light_dir_y, light_dir_z, 
            light_glow, light_bright, light_diffuse_reflec, light_shiny, light_polished, 
            anti_aliasing, new_light_image, transparent_bg)
	
        temp = pdb.gimp_displays_flush()
        
        #Rem Prepare curves
        random.seed() 
        curves_list = [0,0,138,0,161,255,186,0,212,255,235,0,255,255] 
        random_curves_list = [0,0,138,0,161,random.randrange(0,255,1),186,0,212,255,235,0,255,255] 
        #Rem Add curves
    
        if do_sheen:
            pdb.gimp_curves_spline(metal_layer, 0, len(random_curves_list), random_curves_list)
        else:            
            pdb.gimp_curves_spline(metal_layer, 0, len(curves_list), curves_list)        
	
        pdb.gimp_drawable_edit_clear(copy_of_metal)		
		
        if other_layers>0 and not do_gradient:
            pdb.gimp_image_select_color(image, 2, metal_layer, black)
            verdigris_selection = pdb.gimp_selection_save(image)
            pdb.gimp_item_set_name(verdigris_selection,"Verdigris Selection")
            pdb.gimp_selection_none(image)	
	
        copy_layer = pdb.gimp_layer_copy(metal_layer, TRUE)
        temp = pdb.gimp_image_insert_layer(image, copy_layer, None, -1)
        pdb.plug_in_autocrop_layer(image, copy_layer)
        pdb.gimp_drawable_edit_clear(copy_layer)
        copy_layer.fill(1)
        pdb.gimp_image_select_item(image, 2, copy_layer)
        # get y co-ordinate values for blending
        current_selection, cs_x1, cs_y1, cs_x2, cs_y2  = pdb.gimp_selection_bounds(image)
        blend_x0, blend_y0, blend_x1, blend_y1 = fn_gradient_direction(compass, width, cs_x1, cs_y1, cs_x2, cs_y2)
        pdb.gimp_selection_none(image)
        image.remove_layer(copy_layer)
        gradient_mode = fn_set_bevel_mode(effect_mode_ptr)	
	
        # fill the selection with the gradient
        if do_gradient:
            ####fill_gradient = pdb.gimp_context_get_gradient()
            pdb.gimp_context_set_gradient(fill_gradient)
            pdb.gimp_drawable_edit_gradient_fill(copy_of_metal, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
        else:
	        # REM Prepare colours for custom gradient
            #Add colour and layer mode information
            gradient_mode, name_colour = fn_set_metal_colour(custom_colour)
            pdb.gimp_context_set_gradient_fg_bg_rgb()				
            # Do blending
            pdb.gimp_drawable_edit_gradient_fill(copy_of_metal, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
	
        pdb.gimp_item_set_visible(copy_of_metal, True)
        copy_of_metal.mode = gradient_mode
		
        if retain_original:
            gradient_mode = fn_set_bevel_mode(effect_mode_ptr)
            metal_layer.mode = gradient_mode
            pdb.gimp_drawable_desaturate(work_layer,0)
            pdb.gimp_image_select_item(image, 2, metal_layer)
            pdb.gimp_selection_invert(image)
            successful = pdb.gimp_edit_cut(work_layer)
            pdb.gimp_selection_none(image)
        else:
            pdb.gimp_drawable_edit_clear(work_layer)
			
        work_layer = pdb.gimp_image_merge_visible_layers(image,0)
        if final_bump_depth>0: pdb.plug_in_bump_map(image, work_layer, final_copy, effect_azimuth, effect_elevation, final_bump_depth, 0, 0, waterlevel, ambient, True, False, 0)	
        pdb.plug_in_sel_gauss(image, work_layer, artifact_radius, artifact_delta)
		
        image.remove_layer(final_copy)
		
        if other_layers>0 and not do_gradient:
            pdb.gimp_image_select_item(image, 2, verdigris_selection)
            pitted_drawable = pdb.gimp_image_get_layer_by_name(image, pitted_drawable_name)
            success = pdb.gimp_edit_copy(pitted_drawable)
            floater = pdb.gimp_edit_paste(work_layer, False)
            pdb.gimp_floating_sel_anchor(floater)
            image.remove_channel(verdigris_selection)
            layer_id = pdb.gimp_image_get_active_layer(image)
            active_layer = image.active_layer 
			
    if metal_texture == 3: # Dull
	
        if not retain_original:
            temp_layer = gimp.Layer(image, "Temp Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the darker layer for white bases
            image.add_layer(temp_layer, -1)
            metal_grey = fn_get_bg_bump_col(shade_of_grey)
            pdb.gimp_context_set_foreground(metal_grey)
            pdb.gimp_image_select_item(image, 2, work_layer) # get character outline
            pdb.gimp_drawable_edit_fill(temp_layer, 0) # fill character outline
            pdb.gimp_selection_none(image)
            pdb.gimp_drawable_edit_clear(work_layer)
            work_layer = pdb.gimp_image_merge_visible_layers(image,0)
    
	
	
        pdb.gimp_context_set_foreground(black) 
        metal_layer = gimp.Layer(image, "Metal Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the metallic layer
        image.add_layer(metal_layer, -1)
        pdb.gimp_drawable_fill(metal_layer, 0) # fill entire layer
	
        pdb.gimp_image_select_item(image, 2, work_layer)
        # get y co-ordinate values for blending
        current_selection, cs_x1, cs_y1, cs_x2, cs_y2, cs_width, cs_height  = fn_selection_info(image)
        blend_y0 = cs_y1
        blend_y1 = cs_y2
        blend_x0, blend_y0, blend_x1, blend_y1 = fn_gradient_direction(compass, width, cs_x1, cs_y1, cs_x2, cs_y2)
        sel_blur_factor = cs_height * 0.05 # base initial blur on height of actual character within layer
	
        pdb.gimp_context_set_foreground(white)
        pdb.gimp_drawable_edit_fill(metal_layer, 0) # fill character outline
	
        pdb.gimp_selection_none(image)
	
        #bump_depth = bump_depth * 0.65
        #if bump_depth < 1.0: bump_depth = 1.0
        #waterlevel = 0
        #ambient = 0.5
	
	
        # make a copy of the original character so that we can use it later to meld with the newly applied lighting effects
        copy_of_metal = pdb.gimp_layer_copy(metal_layer, True)
        image.add_layer(copy_of_metal, -1)
        pdb.gimp_item_set_visible(copy_of_metal, False)
        pdb.gimp_item_set_name(copy_of_metal,"Copy of Metal")
        pdb.plug_in_gauss(image, copy_of_metal, (sel_blur_factor), (sel_blur_factor), 1)
        final_copy = pdb.gimp_layer_copy(copy_of_metal, True)
        image.add_layer(final_copy, -1)
        pdb.gimp_item_set_name(final_copy,"Final Copy")
		
		
        copy_layer = pdb.gimp_layer_copy(metal_layer, TRUE)
        temp = pdb.gimp_image_insert_layer(image, copy_layer, None, -1)
        pdb.plug_in_autocrop_layer(image, copy_layer)
        pdb.gimp_drawable_edit_clear(copy_layer)
        copy_layer.fill(1)
        pdb.gimp_image_select_item(image, 2, copy_layer)
        # get y co-ordinate values for blending
        current_selection, cs_x1, cs_y1, cs_x2, cs_y2  = pdb.gimp_selection_bounds(image)
        blend_x0, blend_y0, blend_x1, blend_y1 = fn_gradient_direction(compass, width, cs_x1, cs_y1, cs_x2, cs_y2)
        pdb.gimp_selection_none(image)
        image.remove_layer(copy_layer)
		
		
        pdb.gimp_drawable_edit_clear(metal_layer)
		
        if retain_original:
            copy = pdb.gimp_edit_copy(work_layer)
            floater = pdb.gimp_edit_paste(metal_layer ,True)
            pdb.gimp_floating_sel_anchor(floater)
            pdb.gimp_drawable_desaturate(metal_layer,0)			
			
            temp_copy = pdb.gimp_layer_copy(metal_layer, True)
            image.add_layer(temp_copy, -1)
            pdb.gimp_item_set_name(temp_copy,"Temp Copy")
            pdb.gimp_item_set_visible(temp_copy, True)
            random.seed()
            amount = random.randrange(100,10000000,1)			
			
            pdb.gimp_image_select_item(image, 2, work_layer)
            pdb.gimp_context_set_foreground(lightgrey)
            pdb.gimp_context_set_background(verydarkgrey)
            pdb.gimp_context_set_gradient_fg_bg_rgb()
            pdb.gimp_drawable_edit_gradient_fill(metal_layer, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
            random.seed()
		
            pdb.gimp_selection_none(image)
			
			
            mode = 45 # soft light
            temp_copy.mode = mode
            metal_layer = pdb.gimp_image_merge_down(image, temp_copy, 0)
			
			
        else:
            # fill the character outline with colour ready for bumpmapping
            pdb.gimp_image_select_item(image, 2, work_layer)
            pdb.gimp_context_set_foreground(lightgrey)
            pdb.gimp_context_set_background(verydarkgrey)
            pdb.gimp_context_set_gradient_fg_bg_rgb()
            pdb.gimp_drawable_edit_gradient_fill(metal_layer, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
            random.seed()
		
            pdb.gimp_selection_none(image)		
		
        pdb.plug_in_bump_map(image, metal_layer, copy_of_metal, effect_azimuth, effect_elevation, bump_depth, 0, 0, waterlevel, ambient, True, False, 0)		
		
        active_layer = image.active_layer 
        #pdb.plug_in_sel_gauss(image, active_layer, artifact_radius, artifact_delta)
        metal_colour = active_layer
	
	
        # REM: Set lighting effect to metal_layer using copy_of_metal for bump map
        # Set variables for proc_lighting
        bump_map = metal_layer
        #if retain_original: bump_map = metal_layer
        env_map = None
        bump_map_flag = True
        env_map_flag = False
        bump_map_curve = 0
        light_source = 1
        light_colour = (255,255,255)
        #light_pos_x = -1.00
        #light_pos_y = -1.00
        light_pos_z = 1.00
        #light_dir_x = -1.50
        #light_dir_y = -1.50
        light_dir_z = 1.00
        light_glow = 0.80 # 0.60
        light_bright =  0.50 # 0.50
        light_diffuse_reflec = 3.00 #2.80
        light_shiny = 0.20 # 1.00 0.10
        light_polished = 30 #27 20
        anti_aliasing = True
        new_light_image = False
        transparent_bg = False
	
        if apply_lighting:
	        #REM: Call Lighting plugin
            proc_lighting(image, metal_layer, bump_map, env_map, bump_map_flag, env_map_flag, bump_map_curve, light_source, light_colour, 
            light_pos_x, light_pos_y, light_pos_z,
            light_dir_x, light_dir_y, light_dir_z, 
            light_glow, light_bright, light_diffuse_reflec, light_shiny, light_polished, 
            anti_aliasing, new_light_image, transparent_bg)
	
        temp = pdb.gimp_displays_flush()
		
        #Rem Prepare curves
        random.seed()
        curves_list = [0, 0, 34,81,57,21,83,119,103,57,120,181,151,104,156,223,204,67,255, 255]
        random_curves_list = [0, 0, 34,81,57,21,83,119,random.randrange(88,115,1),57,random.randrange(110,140,1),181,151,104,156,223,204,67,255, 255]
        #Rem Add curves
		
        if do_sheen:
            pdb.gimp_curves_spline(metal_layer, 0, len(random_curves_list), random_curves_list)
        else:
            pdb.gimp_curves_spline(metal_layer, 0, len(curves_list), curves_list)
			
			
        pdb.gimp_drawable_edit_clear(copy_of_metal)
		
        pdb.gimp_image_select_item(image, 2, work_layer)
		
        # fill the selection with the gradient
        if do_gradient:
            pdb.gimp_context_set_gradient(fill_gradient)
            pdb.gimp_drawable_edit_gradient_fill(copy_of_metal, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
            gradient_mode = fn_set_bevel_mode(effect_mode_ptr)
        else:
	        # REM Prepare colours for custom gradient
            #Add colour and layer mode information
            gradient_mode, name_colour = fn_set_metal_colour(custom_colour)
            pdb.gimp_context_set_gradient_fg_bg_rgb()				
            # Do blending
            pdb.gimp_drawable_edit_gradient_fill(copy_of_metal, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
			
        pdb.gimp_selection_none(image)
        pdb.gimp_item_set_visible(copy_of_metal, True)
        copy_of_metal.mode = gradient_mode
		
		
        work_layer = pdb.gimp_image_merge_visible_layers(image,0)
        if final_bump_depth>0: pdb.plug_in_bump_map(image, work_layer, final_copy, effect_azimuth, effect_elevation, final_bump_depth, 0, 0, waterlevel, ambient, True, False, 0)	
        pdb.plug_in_sel_gauss(image, work_layer, artifact_radius, artifact_delta)
        image.remove_layer(final_copy)
	
    if metal_texture == 4: # Annealed		
				
        if not retain_original:
            temp_layer = gimp.Layer(image, "Temp Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the darker layer for white bases
            image.add_layer(temp_layer, -1)
            metal_grey = fn_get_bg_bump_col(shade_of_grey)
            pdb.gimp_context_set_foreground(metal_grey)
            pdb.gimp_image_select_item(image, 2, work_layer) # get character outline
            pdb.gimp_drawable_edit_fill(temp_layer, 0) # fill character outline
            pdb.gimp_selection_none(image)
            pdb.gimp_drawable_edit_clear(work_layer)
            work_layer = pdb.gimp_image_merge_visible_layers(image,0)
    
	
	
        pdb.gimp_context_set_foreground(black) 
        metal_layer = gimp.Layer(image, "Metal Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the metallic layer
        image.add_layer(metal_layer, -1)
        pdb.gimp_drawable_fill(metal_layer, 0) # fill entire layer
	
        pdb.gimp_image_select_item(image, 2, work_layer)
        # get y co-ordinate values for blending
        current_selection, cs_x1, cs_y1, cs_x2, cs_y2, cs_width, cs_height  = fn_selection_info(image)
        blend_y0 = cs_y1
        blend_y1 = cs_y2
        sel_blur_factor = cs_height * 0.05 # base initial blur on height of actual character within layer
	
        pdb.gimp_context_set_foreground(white)
        pdb.gimp_drawable_edit_fill(metal_layer, 0) # fill character outline
	
        pdb.gimp_selection_none(image)
	
        #bump_depth = bump_depth * 0.65
        #if bump_depth < 1.0: bump_depth = 1.0
        #waterlevel = 0
        #ambient = 0.5
	
	
        # make a copy of the original character so that we can use it later to meld with the newly applied lighting effects
        copy_of_metal = pdb.gimp_layer_copy(metal_layer, True)
        image.add_layer(copy_of_metal, -1)
        pdb.gimp_item_set_visible(copy_of_metal, False)
        pdb.gimp_item_set_name(copy_of_metal,"Copy of Metal")
        pdb.plug_in_gauss(image, copy_of_metal, (sel_blur_factor), (sel_blur_factor), 1)
        final_copy = pdb.gimp_layer_copy(copy_of_metal, True)
        image.add_layer(final_copy, -1)
        pdb.gimp_item_set_name(final_copy,"Final Copy")
        pdb.gimp_drawable_edit_clear(metal_layer)
	
        if retain_original:
	
            # make a copy of the original character and desaturate it ready for bumpmapping
            copy = pdb.gimp_edit_copy(work_layer)
            floater = pdb.gimp_edit_paste(metal_layer ,True)
            pdb.gimp_floating_sel_anchor(floater)
            pdb.gimp_drawable_desaturate(metal_layer,0)
            temp_copy = pdb.gimp_layer_copy(copy_of_metal, True)
            image.add_layer(temp_copy, -1)
            pdb.gimp_item_set_name(temp_copy,"Temp Copy")
            pdb.gimp_item_set_visible(temp_copy, True)
            random.seed()
            amount = random.randrange(100,10000000,1)
            pdb.plug_in_solid_noise(image,temp_copy, False, True, amount, 15,1,1)
            mode = 45 # soft light
            temp_copy.mode = mode
            metal_layer = pdb.gimp_image_merge_down(image, temp_copy, 0)
		
        else:
        
            # fill the character outline with colour ready for bumpmapping
            pdb.gimp_image_select_item(image, 2, work_layer)
            random.seed()
            amount = random.randrange(100,10000000,1)
            pdb.plug_in_solid_noise(image,metal_layer, False, True, amount, 15,1,1)
            pdb.gimp_drawable_desaturate(metal_layer,0)
            pdb.gimp_selection_none(image)
	
	
        pdb.plug_in_bump_map(image, metal_layer, copy_of_metal, effect_azimuth, effect_elevation, bump_depth, 0, 0, waterlevel, ambient, True, False, 0)
		
        active_layer = image.active_layer 
        #pdb.plug_in_sel_gauss(image, active_layer, artifact_radius, artifact_delta)
        metal_colour = active_layer
	
	
        # REM: Set lighting effect to metal_layer using copy_of_metal for bump map
        # Set variables for proc_lighting
        bump_map = metal_layer
        #if retain_original: bump_map = metal_layer
        env_map = None
        bump_map_flag = True
        env_map_flag = False
        bump_map_curve = 0
        light_source = 1
        light_colour = (255,255,255)
        #light_pos_x = -1.00
        #light_pos_y = -1.00
        light_pos_z = 1.00
        #light_dir_x = -1.50
        #light_dir_y = -1.50
        light_dir_z = 1.00
        light_glow = 0.80 # 0.60
        light_bright =  0.50 # 0.50
        light_diffuse_reflec = 3.00 #2.80
        light_shiny = 0.20 # 1.00 0.10
        light_polished = 30 #27 20
        anti_aliasing = True
        new_light_image = False
        transparent_bg = False
	
        if apply_lighting:
	        #REM: Call Lighting plugin
            proc_lighting(image, metal_layer, bump_map, env_map, bump_map_flag, env_map_flag, bump_map_curve, light_source, light_colour, 
            light_pos_x, light_pos_y, light_pos_z,
            light_dir_x, light_dir_y, light_dir_z, 
            light_glow, light_bright, light_diffuse_reflec, light_shiny, light_polished, 
            anti_aliasing, new_light_image, transparent_bg)
	
        temp = pdb.gimp_displays_flush()
		
        #Rem Prepare curves
        random.seed()
        curves_list = [0,0,44,82,78,31,116,209,141,76,165,255]
        random_curves_list = [0,0,44,82,78,31,116,random.randrange(209,254,1),141,random.randrange(7,76,1),165,255]
        #Rem Add curves
    
        if do_sheen:
            pdb.gimp_curves_spline(metal_layer, 0, len(random_curves_list), random_curves_list)
        else:
            pdb.gimp_curves_spline(metal_layer, 0, len(curves_list), curves_list)
		
	
        pdb.gimp_drawable_edit_clear(copy_of_metal)
	
	
        copy_layer = pdb.gimp_layer_copy(metal_layer, TRUE)
        temp = pdb.gimp_image_insert_layer(image, copy_layer, None, -1)
        pdb.plug_in_autocrop_layer(image, copy_layer)
        pdb.gimp_drawable_edit_clear(copy_layer)
        copy_layer.fill(1)
        pdb.gimp_image_select_item(image, 2, copy_layer)
        # get y co-ordinate values for blending
        current_selection, cs_x1, cs_y1, cs_x2, cs_y2  = pdb.gimp_selection_bounds(image)
        blend_x0, blend_y0, blend_x1, blend_y1 = fn_gradient_direction(compass, width, cs_x1, cs_y1, cs_x2, cs_y2)
        pdb.gimp_selection_none(image)
        image.remove_layer(copy_layer)
        gradient_mode = fn_set_bevel_mode(effect_mode_ptr)	
	
        # fill the selection with the gradient
        if do_gradient:
            pdb.gimp_context_set_gradient(fill_gradient)
            pdb.gimp_drawable_edit_gradient_fill(copy_of_metal, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
        else:
	        # REM Prepare colours for custom gradient
            #Add colour and layer mode information
            gradient_mode, name_colour = fn_set_metal_colour(custom_colour)
            pdb.gimp_context_set_gradient_fg_bg_rgb()				
            # Do blending
            pdb.gimp_drawable_edit_gradient_fill(copy_of_metal, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)	
	
        pdb.gimp_item_set_visible(copy_of_metal, True)
        copy_of_metal.mode = gradient_mode
        pdb.gimp_drawable_edit_clear(work_layer)
        work_layer = pdb.gimp_image_merge_visible_layers(image,0)
        if final_bump_depth>0: pdb.plug_in_bump_map(image, work_layer, final_copy, effect_azimuth, effect_elevation, final_bump_depth, 0, 0, waterlevel, ambient, True, False, 0)	
        pdb.plug_in_sel_gauss(image, work_layer, artifact_radius, artifact_delta)
        image.remove_layer(final_copy)

		
    if metal_texture == 5: # Brushed		
				
        if not retain_original:
            temp_layer = gimp.Layer(image, "Temp Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the darker layer for white bases
            image.add_layer(temp_layer, -1)
            metal_grey = fn_get_bg_bump_col(shade_of_grey)
            pdb.gimp_context_set_foreground(metal_grey)
            pdb.gimp_image_select_item(image, 2, work_layer) # get character outline
            pdb.gimp_drawable_edit_fill(temp_layer, 0) # fill character outline
            pdb.gimp_selection_none(image)
            pdb.gimp_drawable_edit_clear(work_layer)
            work_layer = pdb.gimp_image_merge_visible_layers(image,0)
    
	
	
        pdb.gimp_context_set_foreground(black) 
        metal_layer = gimp.Layer(image, "Metal Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the metallic layer
        image.add_layer(metal_layer, -1)
        pdb.gimp_drawable_fill(metal_layer, 0) # fill entire layer
	
        pdb.gimp_image_select_item(image, 2, work_layer)
        # get y co-ordinate values for blending
        current_selection, cs_x1, cs_y1, cs_x2, cs_y2, cs_width, cs_height  = fn_selection_info(image)
        blend_y0 = cs_y1
        blend_y1 = cs_y2
        sel_blur_factor = cs_height * 0.05 # base initial blur on height of actual character within layer
        pdb.gimp_context_set_foreground(white)
        pdb.gimp_drawable_edit_fill(metal_layer, 0) # fill character outline
	
        pdb.gimp_selection_none(image)
	
        #bump_depth = bump_depth * 0.65
        #if bump_depth < 1.0: bump_depth = 1.0
        #waterlevel = 0
        #ambient = 0.5
	
        # make a copy of the original character so that we can use it later to meld with the newly applied lighting effects
        copy_of_metal = pdb.gimp_layer_copy(metal_layer, True)
        image.add_layer(copy_of_metal, -1)
        pdb.gimp_item_set_visible(copy_of_metal, False)
        pdb.gimp_item_set_name(copy_of_metal,"Copy of Metal")
        pdb.plug_in_gauss(image, copy_of_metal, (sel_blur_factor), (sel_blur_factor), 1)
        final_copy = pdb.gimp_layer_copy(copy_of_metal, True)
        image.add_layer(final_copy, -1)
        pdb.gimp_item_set_name(final_copy,"Final Copy")
        pdb.gimp_drawable_edit_clear(metal_layer)
	
        if retain_original:
	
            # make a copy of the original character and desaturate it ready for bumpmapping
            copy = pdb.gimp_edit_copy(work_layer)
            floater = pdb.gimp_edit_paste(metal_layer ,True)
            pdb.gimp_floating_sel_anchor(floater)
            pdb.gimp_drawable_desaturate(metal_layer,0)			
			
            temp_copy = pdb.gimp_layer_copy(metal_layer, True)
            image.add_layer(temp_copy, -1)
            pdb.gimp_item_set_name(temp_copy,"Temp Copy")
            pdb.gimp_item_set_visible(temp_copy, True)
            random.seed()
            amount = random.randrange(4,20,1)
			
            pdb.gimp_image_select_item(image, 2, work_layer)
            pdb.plug_in_hsv_noise(image,temp_copy, 1,180,255,255)
            pdb.plug_in_hsv_noise(image,temp_copy, 1,180,255,255)
            pdb.plug_in_wind(image,temp_copy, 10,0,amount,1,1)
            pdb.gimp_selection_none(image)
            pdb.gimp_drawable_desaturate(temp_copy,0)
			
            mode = 23 # overlay
            temp_copy.mode = mode
            metal_layer = pdb.gimp_image_merge_down(image, temp_copy, 0)
		
        else:
        
            # fill the character outline with colour ready for bumpmapping
            pdb.gimp_image_select_item(image, 2, work_layer)
            pdb.gimp_context_set_foreground(verylightgrey)
            pdb.gimp_drawable_edit_fill(metal_layer, 0) # fill character outline with grey
            pdb.plug_in_hsv_noise(image,metal_layer, 1,180,255,255)
            pdb.plug_in_hsv_noise(image,metal_layer, 1,180,255,255)
            random.seed()
            amount = random.randrange(4,20,1)
            pdb.plug_in_wind(image,metal_layer, 10,0,amount,0,1)
            pdb.gimp_selection_none(image)
            pdb.gimp_drawable_desaturate(metal_layer,0)
	
        pdb.plug_in_bump_map(image, metal_layer, copy_of_metal, effect_azimuth, effect_elevation, bump_depth, 0, 0, waterlevel, ambient, True, False, 0)
		
        active_layer = image.active_layer 
        if final_bump_depth == 0: pdb.plug_in_sel_gauss(image, active_layer, artifact_radius, artifact_delta)
        metal_colour = active_layer
	
	
        # REM: Set lighting effect to metal_layer using copy_of_metal for bump map
        # Set variables for proc_lighting
        bump_map = metal_layer
        #if retain_original: bump_map = metal_layer
        env_map = None
        bump_map_flag = True
        env_map_flag = False
        bump_map_curve = 0
        light_source = 1
        light_colour = (255,255,255)
        #light_pos_x = -1.00
        #light_pos_y = -1.00
        light_pos_z = 1.00
        #light_dir_x = -1.50
        #light_dir_y = -1.50
        light_dir_z = 1.00
        light_glow = 0.80 # 0.60
        light_bright =  0.50 # 0.50
        light_diffuse_reflec = 3.00 #2.80
        light_shiny = 0.20 # 1.00 0.10
        light_polished = 30 #27 20
        anti_aliasing = True
        new_light_image = False
        transparent_bg = False
	
        if apply_lighting:
	        #REM: Call Lighting plugin
            proc_lighting(image, metal_layer, bump_map, env_map, bump_map_flag, env_map_flag, bump_map_curve, light_source, light_colour, 
            light_pos_x, light_pos_y, light_pos_z,
            light_dir_x, light_dir_y, light_dir_z, 
            light_glow, light_bright, light_diffuse_reflec, light_shiny, light_polished, 
            anti_aliasing, new_light_image, transparent_bg)
	
        temp = pdb.gimp_displays_flush()
		
        #Rem Prepare curves
        random.seed()
        curves_list = [0,0,44,82,78,31,116,209,141,76,165,255]
        random_curves_list = [0,0,44,82,78,31,116,random.randrange(209,254,1),141,random.randrange(7,76,1),165,255]
        #Rem No added curves used for this effect
        pdb.gimp_drawable_edit_clear(copy_of_metal)	
	
        copy_layer = pdb.gimp_layer_copy(metal_layer, TRUE)
        temp = pdb.gimp_image_insert_layer(image, copy_layer, None, -1)
        pdb.plug_in_autocrop_layer(image, copy_layer)
        pdb.gimp_drawable_edit_clear(copy_layer)
        copy_layer.fill(1)
        pdb.gimp_image_select_item(image, 2, copy_layer)
        # get y co-ordinate values for blending
        current_selection, cs_x1, cs_y1, cs_x2, cs_y2  = pdb.gimp_selection_bounds(image)
        blend_x0, blend_y0, blend_x1, blend_y1 = fn_gradient_direction(compass, width, cs_x1, cs_y1, cs_x2, cs_y2)
        pdb.gimp_selection_none(image)
        image.remove_layer(copy_layer)
        gradient_mode = fn_set_bevel_mode(effect_mode_ptr)
	
	
        # fill the selection with the gradient
        if do_gradient:
            pdb.gimp_context_set_gradient(fill_gradient)
            pdb.gimp_drawable_edit_gradient_fill(copy_of_metal, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
        else:
	        # REM Prepare colours for custom gradient
            #Add colour and layer mode information
            gradient_mode, name_colour = fn_set_metal_colour(custom_colour)
            pdb.gimp_context_set_gradient_fg_bg_rgb()
				
            # Do blending
            pdb.gimp_drawable_edit_gradient_fill(copy_of_metal, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
	
        pdb.gimp_item_set_visible(copy_of_metal, True)
        copy_of_metal.mode = gradient_mode
        pdb.gimp_drawable_edit_clear(work_layer)
        work_layer = pdb.gimp_image_merge_visible_layers(image,0)
        if final_bump_depth>0: pdb.plug_in_bump_map(image, work_layer, final_copy, effect_azimuth, effect_elevation, final_bump_depth, 0, 0, waterlevel, ambient, True, False, 0)	
        if final_bump_depth>0: pdb.plug_in_sel_gauss(image, work_layer, artifact_radius, artifact_delta)
		
        image.remove_layer(final_copy)
			
    pdb.gimp_context_set_foreground(current_fg) # re-set foreground colour
	
    active_layer = image.active_layer 
    return active_layer	

        
######################################################################################################################################
  # -------------------------------------------------------------------------------------
	
def proc_lighting(light_image, light_drawable, b_map, e_map, b_map_flag, e_map_flag, b_map_curve, l_source, l_colour, l_pos_x, l_pos_y, l_pos_z, l_dir_x, l_dir_y, l_dir_z, l_glow, l_bright, l_diffuse_reflec, l_shiny, l_polished, a_aliasing, n_image, mk_transp_bg):
    
    # Rem: Use the lighting Plugin
    #LIGHT 1 

    pdb.plug_in_lighting(light_image, light_drawable, b_map, e_map,  b_map_flag, e_map_flag, b_map_curve, l_source, l_colour, l_pos_x, l_pos_y, l_pos_z, l_dir_x, l_dir_y, l_dir_z, l_glow, l_bright, l_diffuse_reflec, l_shiny, l_polished, True, False, False)

    # -------------------------------------------------------------------------------------


        
######################################################################################################################################

def fn_gradient_direction(compass, width, cs_x1, cs_y1, cs_x2, cs_y2):

 
    if compass == 0:
        # North
        blend_x0 = width/2
        blend_y0 = cs_y1
        blend_x1 = width/2
        blend_y1 = cs_y2
    elif compass == 1:
        # North East
        blend_x0 = width
        blend_y0 = cs_y1
        blend_x1 = 0
        blend_y1 = cs_y2
    elif compass == 2:
        # East
        blend_x0 = width
        blend_y0 = cs_y1 + ((cs_y2-cs_y1)/2)
        blend_x1 = 0
        blend_y1 = cs_y1 + ((cs_y2-cs_y1)/2)
    elif compass == 3:
        # South East
        blend_x0 = width
        blend_y0 = cs_y2
        blend_x1 = 0
        blend_y1 = cs_y1
    elif compass == 4:
        # South
        blend_x0 = width/2
        blend_y0 = cs_y2
        blend_x1 = width/2
        blend_y1 = cs_y1
    elif compass == 5:
        # South West
        blend_x0 = 0
        blend_y0 = cs_y2
        blend_x1 = width
        blend_y1 = cs_y1
    elif compass == 6:
        # West
        blend_x0 = 0
        blend_y0 = ((cs_y2-cs_y1)/2)
        blend_x1 = width
        blend_y1 = ((cs_y2-cs_y1)/2)
    else:
        # North West
        blend_x0 = cs_x1
        blend_y0 = cs_y1
        blend_x1 = cs_x2
        blend_y1 = cs_y2
		
    return blend_x0, blend_y0, blend_x1, blend_y1
       
######################################################################################################################################


def fn_compass(compass, light_amount):

    # Set lighting plug-in pos and dir xyz values from compass point selected via dialog UI
    if compass == 0:
        light_pos_x,light_dir_x,light_pos_y,light_dir_y = 0,0,-1,-1 #S
        effect_azimuth = 90
        compass_info = "N"
    elif compass == 1:
        light_pos_x,light_dir_x,light_pos_y,light_dir_y = 1,1,-1,-1 #SW
        effect_azimuth = 45
        compass_info = "NE"
    elif compass == 2:
        light_pos_x,light_dir_x,light_pos_y,light_dir_y = 1,1,0,0 #W
        effect_azimuth = 0
        compass_info = "E"
    elif compass == 3:
        light_pos_x,light_dir_x,light_pos_y,light_dir_y = 1,1,1,1 #NW
        effect_azimuth = 315
        compass_info = "SE"
    elif compass == 4:
        light_pos_x,light_dir_x,light_pos_y,light_dir_y = 0,0,1,1 #N
        effect_azimuth = 270
        compass_info = "S"
    elif compass == 5:
        light_pos_x,light_dir_x,light_pos_y,light_dir_y = -1,-1,1,1 #NE
        effect_azimuth = 225
        compass_info = "SW"
    elif compass == 6:
        light_pos_x,light_dir_x,light_pos_y,light_dir_y = -1,-1,0,0 #E
        effect_azimuth = 180
        compass_info = "W"
    else:
        light_pos_x,light_dir_x,light_pos_y,light_dir_y = -1,-1,-1,-1 #SE
        effect_azimuth = 135
        compass_info = "NW"
		
    # set non UI bumpmap plugin parameters here to reduce height of dialog
    effect_elevation = 35 # possible temp elevation
    #bumpmap_depth = 10 # possible temp bumpmap depth
    waterlevel = 0
    ambient = 0
    
    light_pos_x *= light_amount
    light_pos_y *= light_amount
    light_dir_x *= light_amount
    light_dir_y *= light_amount
	
	
    #light_pos_z, light_dir_z = light_z,light_z
	
    return effect_azimuth, light_pos_x, light_pos_y, light_dir_x, light_dir_y
	
######################################################################################################################################	


def fn_get_bg_bump_col(grey):
    if grey == 0:
        grey_shade = gimpcolor.RGB(222,222,222)
        grey_info = "Lightest grey"
    elif grey == 1:
        grey_shade = gimpcolor.RGB(170, 170, 170)
        grey_info = "Very light grey"
    elif grey == 2:
        grey_shade = gimpcolor.RGB(125, 125, 125)
        grey_info = "Light grey"
    elif grey == 3:
        grey_shade = gimpcolor.RGB(105, 105, 105)
        grey_info = "Mid-grey"
    elif grey == 4:
        grey_shade = gimpcolor.RGB(75, 75, 75)
        grey_info = "Dark grey"
    elif grey == 5:
        grey_shade = gimpcolor.RGB(50, 50, 50)
        grey_info = "Very dark grey"
    else:
        grey_shade = gimpcolor.RGB(20, 20, 20)
        grey_info = "Darkest grey"

    return grey_shade
	
######################################################################################################################################

def fn_set_metal_colour(mc):
    if (mc == 1):
        n_colour = "Gold"
        pdb.gimp_context_set_foreground(lightgold)
        pdb.gimp_context_set_background(darkgold)
        c_mode = 19#SOFTLIGHT
    elif (mc == 2):
        n_colour = "Copper"
        pdb.gimp_context_set_foreground(darkcopper)
        pdb.gimp_context_set_background(lightcopper)
        c_mode = 21#GRAIN MERGE
    elif (mc == 3):
        n_colour = "Brass"
        pdb.gimp_context_set_foreground(darkbrass)
        pdb.gimp_context_set_background(lightbrass)
        c_mode = 19#SOFTLIGHT
    elif (mc == 4):
        n_colour = "Bronze"
        pdb.gimp_context_set_foreground(darkbronze)
        pdb.gimp_context_set_background(lightbronze)
        c_mode = 26#LCH Color new mode
        #c_mode = 45#SOFTLIGHT new mode
    else:
        n_colour = "Silver"
        pdb.gimp_context_set_foreground(lightgrey)
        #pdb.gimp_context_set_background(midgrey)
        pdb.gimp_context_set_background(darkgrey)
        c_mode = 19#SOFTLIGHT

    return c_mode, n_colour
	
######################################################################################################################################
	
def proc_emboss(image, active_layer, black, white, effect_width, effect_elevation, effect_azimuth, emboss_blur, effect_mode_ptr):
    if effect_width < 1: effect_width = 1
    grey = gimpcolor.RGB(80, 80, 80)
    work_layer = active_layer # ie character layer
    work_layer_name = work_layer.name # ie character layer id
    width = work_layer.width
    height = work_layer.height
    colour = gimp.get_background()
    emboss_mode = fn_set_bevel_mode(effect_mode_ptr)
	
    layer_blur = image.height * 0.015
    if layer_blur < 1: layer_blur = 1	
	
    pdb.gimp_context_set_foreground(black)
    pdb.gimp_context_set_background(white)
	
    pdb.gimp_image_select_item(image, 2, work_layer)
	
    emboss_layer = gimp.Layer(image, "Emboss Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the bump map
    image.add_layer(emboss_layer, -1)
    emboss_layer.fill(1)
    pdb.gimp_drawable_edit_fill(emboss_layer, 0)
	
    grey_layer = gimp.Layer(image, "Grey Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the bump map
    image.add_layer(grey_layer, -1)
    pdb.gimp_context_set_foreground(midgrey)	
    #grey_layer.fill(1)
    pdb.gimp_drawable_edit_fill(grey_layer, 0)
	
    pdb.gimp_context_set_foreground(black)
    pdb.gimp_context_set_background(white)
	
    pdb.gimp_selection_none(image) ####
    pdb.plug_in_gauss_iir(image, emboss_layer, emboss_blur, True, True) # blur the bump map
	
    other_layers = 1
    bumpmap_layer_name = "Emboss Layer"
    bumpmap_layer = emboss_layer
	
    #active_layer = new_layer
	
    #temp values
    bump_waterlevel = 0
    bump_ambient = 0
    bump_depth = effect_width
    waterlevel = 0
    ambient = 0
    #end temp values
	
    if other_layers > 0:
        work_layer = active_layer
        work_layer_name = work_layer.name
        width = work_layer.width
        height = work_layer.height
        #bumpmap_width = bumpmap_width * 0.65
        #if bumpmap_width < 1: bumpmap_width = 1
		
        if waterlevel > 0:
            waterlevel = waterlevel/100
            waterlevel = 1.0 - waterlevel
        else:
            waterlevel = 0.0
			
        if ambient > 0:
            ambient = ambient/100
            ambient = 1.0 - ambient
        else:
            ambient = 0.0        

        pdb.plug_in_bump_map(image, grey_layer, emboss_layer, effect_azimuth, effect_elevation, bump_depth, 0, 0, waterlevel, ambient, True, False, 0)
    #work_layer = image.active_layer
	
    image.remove_layer(emboss_layer)
    image.lower_layer(grey_layer)
    #temp values
    blur_radius = 10
    blur_delta = 8
    #end temp values
    pdb.plug_in_sel_gauss(image, grey_layer, blur_radius, blur_delta)
    work_layer.mode = emboss_mode
    pdb.gimp_image_merge_down(image, work_layer, 0)
    work_layer = image.active_layer
    pdb.gimp_item_set_name(work_layer, work_layer_name)
    #image.remove_layer(grey_layer)
		
    pdb.gimp_selection_none(image)
    active_layer = image.active_layer 
    return active_layer	
	
######################################################################################################################################
	
def fn_set_bevel_mode(bevel_mode_ptr):
    # Use New Modes when creating a bevel
    new_mode = bevel_mode_ptr
    temp_mode = 47
    if new_mode == 0:
        temp_mode = 0
        n_mode_info = "Normal"
    elif new_mode == 1:
        temp_mode = 1
        n_mode_info = "Dissolve"
    elif new_mode == 2:
        temp_mode = 57
        _mode_info = "Color Erase"
    elif new_mode == 3:
        temp_mode = 59
        n_mode_info = "Merge"
    elif new_mode == 4:
        temp_mode = 60
        n_mode_info = "Split"
    elif new_mode == 5:
        temp_mode = 36
        n_mode_info = "Lighten Only"
    elif new_mode == 6:
        temp_mode = 55
        n_mode_info = "Luma Luminance Lighten"
    elif new_mode == 7:
        temp_mode = 31
        n_mode_info = "Screen"
    elif new_mode == 8:
        temp_mode = 42
        n_mode_info = "Dodge"
    elif new_mode == 9:
        temp_mode = 33
        n_mode_info = "Addition"
    elif new_mode == 10:
        temp_mode = 35
        n_mode_info = "Darken only"
    elif new_mode == 11:
        temp_mode = 54
        n_mode_info = "Luma Luminance darken"
    elif new_mode == 12:
        temp_mode = 30
        n_mode_info = "Multiply"
    elif new_mode == 13:
        temp_mode = 43
        n_mode_info = "Burn"
    elif new_mode == 14:
        temp_mode = 53
        n_mode_info = "Linear burn"
    elif new_mode == 15:
        temp_mode = 23
        n_mode_info = "Overlay"
    elif new_mode == 16:
        temp_mode = 45
        n_mode_info = "Soft light"
    elif new_mode == 17:
        temp_mode = 44
        n_mode_info = "Hard light"
    elif new_mode == 18:
        temp_mode = 48
        n_mode_info = "Vivid light"
    elif new_mode == 19:
        temp_mode = 49
        n_mode_info = "Pin light"
    elif new_mode == 20:
        temp_mode = 50
        n_mode_info = "Linear light"
    elif new_mode == 21:
        temp_mode = 51
        n_mode_info = "Hard mix"
    elif new_mode == 22:
        temp_mode = 32
        n_mode_info = "Difference"
    elif new_mode == 23:
        temp_mode = 52
        n_mode_info = "Exclusion"
    elif new_mode == 24:
        temp_mode = 34
        n_mode_info = "Subtract"
    elif new_mode == 25:
        temp_mode = 46
        n_mode_info = "Grain extract"
    elif new_mode == 26:
        temp_mode = 47
        n_mode_info = "Grain merge"
    elif new_mode == 27:
        temp_mode = 41
        n_mode_info = "Divide"
    elif new_mode == 28:
        temp_mode = 37
        n_mode_info = "Hue HSV"
    elif new_mode == 29:
        temp_mode = 38
        n_mode_info = "Saturation HSV"
    elif new_mode == 30:
        temp_mode = 39
        n_mode_info = "Color HSL"
    elif new_mode == 31:
        temp_mode = 40
        n_mode_info = "Value HSL"
    elif new_mode == 32:
        temp_mode = 24
        n_mode_info = "Hue LCH"
    elif new_mode == 33:
        temp_mode = 25
        n_mode_info = "Chroma LCH"
    elif new_mode == 34:
        temp_mode = 26
        n_mode_info = "Color LCH"
    elif new_mode == 35:
        temp_mode = 27
        n_mode_info = "Lightness LCH"
    elif new_mode == 36:
        temp_mode = 56
        n_mode_info = "Luminance"
		
		
    bevel_mode = temp_mode
    return bevel_mode




######################################################################################################################################

def fn_test_for_custom_layer(img,l_id,l_name):
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5:
        if ".png" in l_name:
            test = True
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################

######################################################################################################################################

    #### Useful Functions
# -------------------------------------------------------------------------------------
	
def fn_selection_info(im):
    # returns selection info
    selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(im)
    s_width, s_height = x2 - x1, y2 - y1
	
    return selection, x1, x2, y1, y2, s_width, s_height


######################################################################################################################################
	
def fn_selection_amount(sel_value,r_ness,adjustment):
    # 
    value = (math.sqrt(sel_value)*r_ness)*adjustment
    return value

######################################################################################################################################

def fn_set_metal_colour2(mc):
    # Returns mode for metallic colours and the colour name
    if (mc == 1):
        n_colour = "Gold"
        pdb.gimp_context_set_foreground(lightgold)
        pdb.gimp_context_set_background(darkgold)
        c_mode = 45#SOFTLIGHT new mode
    elif (mc == 2):
        n_colour = "Copper"
        pdb.gimp_context_set_foreground(darkcopper)
        pdb.gimp_context_set_background(lightcopper)
        c_mode = 47#GRAIN MERGE new mode
    elif (mc == 3):
        n_colour = "Brass"
        pdb.gimp_context_set_foreground(darkbrass)
        pdb.gimp_context_set_background(lightbrass)
        c_mode = 45#SOFTLIGHT new mode
    elif (mc == 4):
        n_colour = "Bronze"
        pdb.gimp_context_set_foreground(darkbronze)
        pdb.gimp_context_set_background(lightbronze)
        c_mode = 26#LCH Color new mode
        #c_mode = 45#SOFTLIGHT new mode
    else:
        n_colour = "Silver"
        pdb.gimp_context_set_foreground(lightgrey)
        pdb.gimp_context_set_background(midgrey)
        c_mode = 45#SOFTLIGHT new mode

    return c_mode, n_colour

######################################################################################################################################
	
def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)
	
######################################################################################################################################
		
def proc_trim_away(img, layer, amount):
    #### Trim off pixels from a layer
    # set our feather settings
    our_feather = True
    our_feather_radius = 2
    #get feather settings
    feather = pdb.gimp_context_get_feather()
    feather_x, feather_y = pdb.gimp_context_get_feather_radius()
    # make sure that feather is set
    if not feather: pdb.gimp_context_set_feather(our_feather)
    # use our feather radius
    pdb.gimp_context_set_feather_radius(our_feather_radius, our_feather_radius)
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    # reset feather settings
    pdb.gimp_context_set_feather(feather)
    pdb.gimp_context_set_feather_radius(feather_x, feather_y)
	
    #### End of Proc_trim_away
	
######################################################################################################################################
	
def proc_text_bevel(image, text_layer, bevel_width, bevel_depth, feather, black, white, midgrey, bevel_mode):
    #### Create a bevel around text and return the new layer as bevel_layer
    #Rem Select text outline and save it
    pdb.gimp_image_select_item(image, 2, text_layer)
    saved_selection = pdb.gimp_selection_save(image)

    #### Rem Make a bumpmap layer ######
    bumpmap_layer = pdb.gimp_layer_new_from_drawable(text_layer, image)
    pdb.gimp_image_insert_layer(image, bumpmap_layer, None, -1)
    pdb.gimp_item_set_name(bumpmap_layer, "Text Layer Bumpmap")
    pdb.gimp_image_raise_item_to_top(image, bumpmap_layer)

    pdb.gimp_selection_feather(image, feather)
    blurred_selection = pdb.gimp_selection_save(image)
    loop = bevel_width
    pdb.gimp_selection_shrink(image, loop)

    while (pdb.gimp_selection_is_empty(image) == True ):
        loop = loop - 1
        pdb.gimp_image_select_item(image, 2, blurred_selection)
        pdb.gimp_selection_shrink(image, loop)
        
    bevel_width = min(bevel_width, loop)
    pdb.gimp_image_select_item(image, 2, blurred_selection)

    pdb.gimp_context_set_foreground(black)
    pdb.gimp_drawable_fill(bumpmap_layer, 0)

    loop = 1
    #Rem Temp dirty fill_value 
    fill_value = (255 / bevel_width)
    while (loop <= bevel_width):

        #Rem Using Temp fill_value until something better comes to mind
        pdb.gimp_image_select_item(image, 2, blurred_selection)
        pdb.gimp_selection_shrink(image, loop -1)
        greycolour = int(fill_value)
        grey = gimpcolor.RGB(greycolour, greycolour, greycolour)
        pdb.gimp_context_set_foreground(grey)

        pdb.gimp_edit_fill(bumpmap_layer, 0)

        #Rem Temp fill_value
        fill_value = fill_value + (255 / bevel_width)
        loop = loop + 1

    pdb.gimp_context_set_foreground(white)
    pdb.gimp_image_select_item(image, 2, blurred_selection)
    pdb.gimp_selection_shrink(image, bevel_width)

    if (pdb.gimp_selection_is_empty(image) == FALSE):
        pdb.gimp_edit_fill(bumpmap_layer, 0)

    pdb.gimp_selection_none(image)

    #### Make Bevel from bumpmap ####
    bevel_layer = pdb.gimp_layer_new_from_drawable(text_layer, image)
    pdb.gimp_image_insert_layer(image, bevel_layer, None, -1)
    pdb.gimp_item_set_name(bevel_layer, "Bevel")
    pdb.gimp_image_raise_item_to_top(image, bevel_layer)

    pdb.gimp_context_set_foreground(midgrey)
    pdb.gimp_drawable_fill(bevel_layer, 0)
	
    pdb.plug_in_bump_map(image, bevel_layer, bumpmap_layer, 135, 35, bevel_depth, 0, 0, 0, 0, True, False, 0)
    pdb.gimp_layer_set_mode(bevel_layer, bevel_mode)
    pdb.gimp_layer_set_opacity(bevel_layer, 80)
	
    #### Delete area outside the bevel ####
    pdb.gimp_image_select_item(image, 2, saved_selection)
    pdb.gimp_selection_invert(image)

    if (pdb.gimp_selection_is_empty(image) == False):
        pdb.gimp_edit_clear(bevel_layer)

    pdb.gimp_image_select_item(image, 2, saved_selection)
    pdb.gimp_selection_shrink(image, bevel_width)

    if (pdb.gimp_selection_is_empty(image) == False):
        pdb.gimp_edit_clear(bevel_layer)
	
    pdb.gimp_image_select_item(image, 2, saved_selection)
 
    pdb.gimp_selection_none(image)
    pdb.gimp_drawable_set_visible(bumpmap_layer, False)	
    pdb.gimp_image_remove_channel(image, saved_selection)
    pdb.gimp_image_remove_channel(image, blurred_selection)
    pdb.gimp_image_remove_layer(image, bumpmap_layer)
    pdb.gimp_image_raise_item_to_top(image, bevel_layer)
    pdb.gimp_item_set_linked(bevel_layer, True)
    return bevel_layer
	
#### End of proc_text_bevel

######################################################################################################################################
	
register(
    "python_fu_Custom_Font_Metallic_Effects_v0-01",
    "Provides options for applying metallic effects to Gimp Custom Fonts",
    "Provides options for applying metallic effects to Gimp Custom Fonts",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2019",
    "<Image>/Filters/Custom Font Tools/Custom Font Metallic Effects v0.01...",
    "RGB*",
    [
    (PF_OPTION, "pf_option_texture", "Metal texture", 1,("Glitter","Bright","Verdigris","Dull","Annealed","Brushed")),
    (PF_OPTION, "pf_option_colour", "Metallic Colour", 1,("Silver","Gold","Copper", "Brass", "Bronze")),
    (PF_TOGGLE, "pf_usertextureoff",   "Employ original texture", 0),
    (PF_TOGGLE, "pf_stroke_gradient", "Replace metallic with current gradient: ", 0),
    (PF_GRADIENT, "pf_current_gradient", "Current gradient:", 0),
    (PF_SPINNER, "pf_bump_depth", "Bumpmap depth (as percentage): ", 30,(0,100,1)),
    (PF_SPINNER, "pf_light_amount", "Distance of light source\nfrom layer centre: ", 1.50,(0.00,1000.00,0.10)),
    (PF_OPTION, "pf_compass", "Point light position or\nlight direction as compass point: ", 7,("N","NE","E","SE","S","SW","W","NW")),
    (PF_SPINNER, "pf_effect_elevation", "Effect elevation: ", 30.0,(0.5,90,0.1)),
    (PF_TOGGLE, "pf_userandomsheenonoff",   "Randomize sheen", 0),
    (PF_TOGGLE, "pf_userlightingoff",   "Apply lighting plug-in", 0),
    (PF_OPTION, "pf_base_colour", "Shade of grey to use\nwith plain base characters: ", 3,("Lightest grey" ,"Very light grey","Light grey","Mid-grey","Dark grey","Very dark grey","Darkest grey")),
    (PF_OPTION, "mode_option2", "Metal colour mode: ", 26, ("Normal", "Dissolve","Color erase", "Merge", "Split", "Lighten only", "Luma/Luminance lighten only", "Screen", "Dodge", "Addition", "Darken only", "Luma/Luminance darken only", "Multiply", "Burn", "Linear burn", "Overlay", "Soft light", "Hard light", "Vivid light", "Pin light", "Linear light", "Hard mix", "Difference", "Exclusion", "Subtract", "Grain extract", "Grain merge", "Divide", "Hue (HSV)", "Saturation (HSV)", "Color (HSL)", "Value (HSL)", "Hue (LCH)", "Chroma (LCH)", "Color (LCH)", "Lightness (LCH)", "Luminance")),
    (PF_SPINNER, "pf_blur_radius", "Artifact blur radius: ", 10,(0.0,100.0,0.1)),
    (PF_SPINNER, "pf_blur_delta", "Artifact blur delta amount: ", 150,(0,255,1)),
    (PF_SPINNER, "pf_final_bump_depth", "Final bumpmap depth (as percentage): ", 0,(0,100,1)),
    ],
    [],
    do_stuff)

main()
